@@ -5,7 +5,7 @@ use warnings;
 
 use parent qw(Tickit::Widget);
 
-our $VERSION = '0.209';
+our $VERSION = '0.210';
 
 =head1 NAME
 
@@ -13,7 +13,7 @@ Tickit::Widget::Table - table widget with support for scrolling/paging
 
 =head1 VERSION
 
-version 0.209
+version 0.210
 
 =head1 SYNOPSIS
 
@@ -259,6 +259,8 @@ sub new {
 		highlight_row
 		data
 	);
+	$attr{header_visible} //= 1;
+	$attr{header_lines} //= 1;
 	my $self = $class->SUPER::new(@_);
 
 	# First we assign the adapter, since it might be used elsewhere
@@ -521,8 +523,8 @@ Returns the L<Tickit::Rect> representing the body area.
 sub body_rect {
 	my $self = shift;
 	$self->{body_rect} ||= Tickit::Rect->new(
-		top   => 1,
-		lines => $self->window->lines - 1,
+		top   => $self->header_lines,
+		bottom => $self->window->lines - 1,
 		left  => 0,
 		cols  => $self->window->cols - 1
 	);
@@ -537,20 +539,56 @@ Returns the L<Tickit::Rect> representing the scroll bar.
 sub scrollbar_rect {
 	my $self = shift;
 	$self->{scrollbar_rect} ||= Tickit::Rect->new(
-		top   => 1,
-		lines => $self->window->lines - 1,
+		top   => $self->header_lines,
+		bottom => $self->window->lines - 1,
 		left  => $self->window->cols - 1,
 		cols  => 1,
 	);
 }
 
+=head2 hide_header
+
+Removes the header - the body will expand upwards to compensate.
+.
+=cut
+
+sub hide_header {
+	my ($self) = @_;
+	$self->window->expose if $self->window;
+	delete @{$self}{qw(body_rect header_rect scrollbar_rect)};
+	$self->{header_visible} = 0;
+	$self
+}
+
+=head2 show_header
+
+Makes the header visible again. See L</hide_header>.
+
+=cut
+
+sub show_header {
+	my ($self) = @_;
+	$self->{header_visible} = 1;
+	delete @{$self}{qw(body_rect header_rect scrollbar_rect)};
+	$self->window->expose if $self->window;
+	$self
+}
+
+=head2 header_visible
+
+Returns true if the header is visible, 0 otherwise.
+
+=cut
+
+sub header_visible { $_[0]{header_visible} ? 1 : 0 }
+
 =head2 header_lines
 
 Returns the number of lines in the header. Hardcoded to 1.
 
 =cut
 
-sub header_lines { 1 }
+sub header_lines { $_[0]->header_visible ? $_[0]->{header_lines} : 0 }
 
 =head2 body_lines
 
@@ -650,10 +688,10 @@ sub render_to_rb {
 	my $win = $self->window;
 	$self->{highlight_row} ||= 0;
 
-	$self->render_header($rb, $rect);
+	$self->render_header($rb, $rect) if $self->header_visible;
 	$self->render_body($rb, $rect);
 	$self->render_scrollbar($rb, $rect) if $self->vscroll;
-	my $highlight_pos = 1 + $self->highlight_visible_row;
+	my $highlight_pos = $self->header_lines + $self->highlight_visible_row;
 	$win->cursor_at($highlight_pos, 0);
 }
 
@@ -709,26 +747,26 @@ sub render_scrollbar {
 		or return $self;
 
 	my $cols = $win->cols - 1;
-	my $h = $win->lines - 1;
+	my $h = $win->lines - $self->header_lines;
 
 	# Need to clear any line content first, since we may be overwriting part of
 	# the previous scrollbar rendering here
 	$rb->eraserect(
 		Tickit::Rect->new(
-			top => 1,
+			top => $self->header_lines,
 			left => $cols,
 			right => $cols,
 			bottom => $h,
 		)
 	);
-	if(my ($min, $max) = map 1 + $_, $self->scroll_rows) {
+	if(my ($min, $max) = map $self->header_lines + $_, $self->scroll_rows) {
 		# Scrollbar should be shown, since we don't have all rows visible on the screen at once
-		$rb->vline_at(1, $min - 1, $cols, LINE_SINGLE, $self->get_style_pen('scrollbar'), CAP_BOTH) if $min > 1;
+		$rb->vline_at($self->header_lines, $min - 1, $cols, LINE_SINGLE, $self->get_style_pen('scrollbar'), CAP_BOTH) if $min > 1;
 		$rb->vline_at($min, $max, $cols, LINE_DOUBLE, $self->get_style_pen('scroll'), CAP_BOTH);
 		$rb->vline_at($max + 1, $h, $cols, LINE_SINGLE, $self->get_style_pen('scrollbar'), CAP_BOTH) if $max < $h;
 	} else {
 		# Placeholder scrollbar - just render it as empty
-		$rb->vline_at(1, $h, $cols, LINE_SINGLE, $self->get_style_pen('scrollbar'), CAP_BOTH);
+		$rb->vline_at($self->header_lines, $h, $cols, LINE_SINGLE, $self->get_style_pen('scrollbar'), CAP_BOTH);
 	}
 }
 
@@ -781,7 +819,7 @@ sub render_row {
 	my $base_pen = $self->get_style_pen(
 		($row == $self->highlight_row)
 		? 'highlight'
-		: ($self->multi_select && $self->{selected}{$line + $self->row_offset - 1})
+		: ($self->multi_select && $self->{selected}{$line + $self->row_offset - $self->header_lines})
 		? 'selected'
 		: undef
 	);
@@ -820,7 +858,7 @@ sub render_failed_row {
 	my $base_pen = $self->get_style_pen(
 		($row == $self->highlight_row)
 		? 'highlight'
-		: ($self->multi_select && $self->{selected}{$line + $self->row_offset - 1})
+		: ($self->multi_select && $self->{selected}{$line + $self->row_offset - $self->header_lines})
 		? 'selected'
 		: 'failed'
 	);
@@ -838,7 +876,7 @@ sub render_pending_row {
 	my $base_pen = $self->get_style_pen(
 		($row == $self->highlight_row)
 		? 'highlight'
-		: ($self->multi_select && $self->{selected}{$line + $self->row_offset - 1})
+		: ($self->multi_select && $self->{selected}{$line + $self->row_offset - $self->header_lines})
 		? 'selected'
 		: 'pending'
 	);
@@ -1168,7 +1206,7 @@ sub active_scrollbar_rect {
 	my $self = shift;
 	return unless my ($start, $end) = $self->scroll_rows;
 	Tickit::Rect->new(
-		top => 1 + $start,
+		top => $self->header_lines + $start,
 		bottom => 2 + $end,
 		left => $self->window->cols - 1,
 		cols => 1,
@@ -1558,6 +1596,8 @@ Current list of pending features:
 
 =item * Better header support (more than one row, embedded widgets)
 
+=item * More efficient redraw when showing/hiding header (scroll body and redraw just the header lines)
+
 =back
 
 =head1 SEE ALSO
